import sys
import pickle
import uproot
import pandas
from xgboost.sklearn import XGBClassifier
import matplotlib.pyplot as plt

# Besser: Einlesen mit argparse
infile  = sys.argv[1]
outfile = sys.argv[2]
print(infile, outfile)

print("Loading bdt model")
bdt = pickle.load(open("bdt.model", "rb"))

print("Loading data")
data = uproot.open(infile)["DecayTree"].arrays(outputtype=pandas.DataFrame)

print("Applying bdt")
data["bdt"] = bdt.predict_proba(data[["f1", "f2", "f3"]])[:,1]

print("Writing output file")
with uproot.recreate(outfile) as out:
    out["DecayTree"] = uproot.newtree(data.dtypes)
    out["DecayTree"].extend(data.to_dict("list"))

print("Drawing some more nice plots")
plt.figure(figsize=(10,10))
plt.subplot(2, 2, 1)
plt.title("BDT output distribution")
plt.hist(data.bdt, bins=100, range=(0, 1))
plt.subplot(2, 2, 2)
plt.title("Full dataset")
plt.hist2d(data.f1, data.f2, bins=[100, 100], cmin=1, range=[[-3,3],[-4,4]])
plt.xlabel("f1")
plt.ylabel("f2")
plt.subplot(2, 2, 3)
plt.title("Dataset with bdt>0.5 cut")
plt.hist2d(data[data.bdt>0.5].f1, data[data.bdt>0.5].f2, bins=[100, 100], cmin=1, range=[[-3,3],[-4,4]])
plt.xlabel("f1")
plt.ylabel("f2")
plt.subplot(2, 2, 4)
plt.title("Dataset with bdt<0.5 cut")
plt.hist2d(data[data.bdt<0.5].f1, data[data.bdt<0.5].f2, bins=[100, 100], cmin=1, range=[[-3,3],[-4,4]])
plt.xlabel("f1")
plt.ylabel("f2")

plt.savefig(f"{infile.replace('.root','')}_classification.pdf")

