import uproot
import pandas
import numpy
import matplotlib.pyplot as plt

def generate_2d_gaussian(mu, s, rho, label=None, size=20000):
    data = numpy.random.multivariate_normal(mean=mu, cov=[[s[0], rho],[rho, s[1]]], size=20000)
    if label is not None:
        return pandas.DataFrame({"f1" : data[:,0], "f2" : data[:,1], "label" : numpy.ones(len(data))*label})
    else:
        return pandas.DataFrame({"f1" : data[:,0], "f2" : data[:,1]})

print("Creating labeled training file")
data_bkg = generate_2d_gaussian([0,0], [0.5, 0.8], 1, label=0, size=60000)
data_sig = generate_2d_gaussian([0.5,-1.5], [0.05, 0.5], -0.5, label=1, size=10000)
data_sig_2 = generate_2d_gaussian([1,2.5], [0.05, 0.5], +0.5, label=1, size=20000)

plt.plot(data_bkg.f1, data_bkg.f2, 'r,', label="background")
plt.plot(data_sig.f1, data_sig.f2, 'b,', label="signal")
plt.plot(data_sig_2.f1, data_sig_2.f2, 'b,')
plt.legend()
plt.xlim(-5,5)
plt.ylim(-5,5)
plt.savefig("input.pdf")
plt.show()

training_data = pandas.concat([data_bkg, data_sig, data_sig_2])
training_data.reset_index(drop=True, inplace=True)
training_data = training_data.sample(frac=1)

print("Creating data-like file")
real_data_bkg = generate_2d_gaussian([0,0], [0.5, 0.8], 1)
real_data_sig = generate_2d_gaussian([0.5,-1.5], [0.05, 0.5], -0.5)
real_data_sig2 = generate_2d_gaussian([1,2.5], [0.05, 0.5], +0.5)

dataset = pandas.concat([real_data_bkg, real_data_sig, real_data_sig2])
dataset.reset_index(drop=True, inplace=True)
dataset = dataset.sample(frac=1)

with uproot.recreate("mc_raw.root") as mcfile:
    mcfile["DecayTree"] = uproot.newtree({"f1" : float, "f2" : float, "label" : int})
    mcfile["DecayTree"].extend(training_data.to_dict("list"))

with uproot.recreate("data_raw.root") as datafile:
    datafile["DecayTree"] = uproot.newtree({f : float for f in ["f1", "f2"]})
    datafile["DecayTree"].extend(dataset.to_dict("list"))
